package Language::Messages;

use strict;

sub new {
    my ( $class, $message_hash_ref ) = @_;
    my $self = {};

    bless $self, $class;
    $self->{message_hash_ref} = $message_hash_ref;

    return $self;
}

sub get_message {
    my ( $self, $key, @params ) = @_;

    return 'Language::Messages::get_message(): Key required!'
        if not defined $key;

    my $message = $self->{message_hash_ref}->{$key};

    return "Language::Messages::get_message(): No message defined for '$key'!"
        if not defined $message;

    my $index = 1;
    foreach my $param (@params) {
        my $subst = "%${index}";
        $message =~ s/$subst/$param/g;
        $index++;
    }
    return $message;
}

1;

__END__

=head1 NAME

Language::Messages - Liefert (sprachspezifische) Texte

=head1 DESCRIPTION

Die Klasse C<Language::Messages> liefert (sprachspezifische) Texte.

=head1 METHODS

=over

=item $messages = Language::Messages->new(%hash_ref)

Erzeugt ein Objekt.

Der 1. Parameter ist ein Hash mit den Schlsseln und Texten (Messagemap)

=item $messages->get_message($key, @params)

Liefert eine Nachricht aus der Messagemap. Dabei werden alle Vorkommen
des Strings '%i' durch einen Parameter ersetzt: '%1' durch den ersten,
'%2' durch den zweiten usw.

Der erste Parameter ist ein String mit dem Schlssel

Die folgenden Parameter sind optionale Parameter, die in den Messagestring
eingesetzt werden in '%1', '%2' usw.

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/10/27

=cut
