package Util::File;

use version; our $VERSION = qv('0.1_0');
use strict;

sub touch_reference {
    my ( $filename, $reference_filename ) = @_;
    my ( $atime, $mtime ) = ( stat($reference_filename) )[ 8, 9 ];

    utime $atime, $mtime, $filename;
}

sub is_newer {
    my ( $filename_1, $filename_2 ) = @_;
    my $time1 = ( stat($filename_1) )[9];
    my $time2 = ( stat($filename_2) )[9];

    return $time1 > $time2;
}

sub equal_times {
    my ( $filename_1, $filename_2 ) = @_;
    my $time1 = ( stat($filename_1) )[9];
    my $time2 = ( stat($filename_2) )[9];

    return $time1 == $time2;
}

1;

__END__

=head1 NAME

Util::File - Utilities fr Dateien

=head1 DESCRIPTION

Die Klasse Util::File bietet Funktionen an fr die Arbeit mit Dateien.

=head1 METHODS

=over

=item Util::File::touch_reference($filename, $reference_filename)

Setzt die Zeiten des letzten Zugriffs und der letzten Modifikation einer Datei
auf gleich denen einer anderen Datei, so wie C<touch --reference=FILE>.

Der 1. Parameter ist ein String mit dem Namen der Datei, deren Zeiten gesetzt
werden

Der 2. Parameter ist ein String mit dem Namen der Refernzdatei, deren Zeiten
bernommen werden

=item $is_newer = Util::File::is_newer($filename_1, $filename_2)

Liefert, ob eine Datei neuer ist als eine andere. Verglichen werden
die letzten nderungszeiten (C<stat()[9]>).

Der 1. Parameter ist ein String mit dem Namen der ersten Datei

Der 2. Parameter ist ein String mit dem Namen der zweiten Datei

Die Rckgabe ist true, wenn die erste Datei C<$filename_1> neuer ist als die
zweite Datei C<$filename_2>

=item $equal_times = Util::File::equal_times($filename_1, $filename_2)

Liefert, ob zwei Dateien die gleichen Zeiten haben. Verglichen werden
die letzten nderungszeiten (C<stat()[9]>).

Der 1. Parameter ist ein String mit dem Namen der ersten Datei

Der 2. Parameter ist ein String mit dem Namen der zweiten Datei

Die Rckgabe ist true, wenn beide Dateien die gleiche Zeit haben

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/09/23

=cut
