package Util::Object;

use version; our $VERSION = qv('0.2_0');
use strict;

use Util::Array;

sub equals {
    my ( $object_1_ref, $object_2_ref ) = @_;

    return 1
        if ( ( not defined $object_1_ref ) && ( not defined $object_2_ref ) );
    return 0
        if ( ( not defined $object_1_ref ) || ( not defined $object_2_ref ) );
    return 0
        if !_assert_references( $object_1_ref, $object_2_ref );
    return 0
        if !_assert_types_equals( $object_1_ref, $object_2_ref );
    return 1 if ( $object_1_ref == $object_2_ref );

    if ( ref $object_1_ref eq 'ARRAY' ) {
        return Util::Array::elements_equals( $object_1_ref, $object_2_ref );
    }
    elsif ( ref $object_1_ref eq 'SCALAR' ) {
        return ${$object_1_ref} eq ${$object_2_ref};
    }
    elsif ( ref $object_1_ref eq 'HASH' ) {
        return 0
            if _get_key_count($object_1_ref) != _get_key_count($object_2_ref);
        foreach my $key ( keys %{$object_1_ref} ) {
            if ( ref( $object_1_ref->{$key} ) ) {
                return 0
                    if !Util::Object::equals( $object_1_ref->{$key},
                    $object_2_ref->{$key} );
            }
            else {
                return 0
                    if !Util::Object::equals( \$object_1_ref->{$key},
                    \$object_2_ref->{$key} );
            }
        }
        return 1;
    }
    else {
        print STDERR 'Util::Object::equals(): Objekttyp ', ref $object_1_ref,
            ' wird nicht unterstuetzt!';
    }

    return 0;
}

sub _assert_references {
    my @argv = @_;

    my $argc = @argv;
    return 0 if $argc == 0;
    return 1 if $argc == 1 && ref $argv[0];

    my $prev_arg = $argv[0];
    foreach my $arg (@argv) {
        if ( !ref($prev_arg) || !ref($arg) ) {
            print STDERR
                "Argumente '$prev_arg' und '$arg' sind keine Referenzen!\n";
            return 0;
        }
    }
    return 1;
}

sub _assert_types_equals {
    my @argv = @_;

    my $argc = @argv;
    return 0 if $argc == 0;
    return 1 if $argc == 1;

    my $prev_arg = $argv[0];
    foreach my $arg (@argv) {
        my $prev_type = _get_type($prev_arg);
        my $type      = _get_type($arg);
        if ( $prev_type ne $type ) {
            print STDERR "Typen sind verschieden "
                . "['$prev_type' != '$type' ('$prev_arg', '$arg')]! ";
            return 0;
        }
    }
    return 1;
}

sub _get_type {
    my ($arg) = @_;

    return ref $arg ? ref $arg : ref \$arg;
}

sub _get_key_count {
    my ($hash_ref)    = @_;
    my @keys_hash_ref = keys %{$hash_ref};
    my $key_count     = @keys_hash_ref;

    return $key_count;
}

sub is_type {
    my ( $type, $param ) = @_;
    
    return ref $param ? ref $param eq $type : ref \$param eq $type;
}

1;

__END__

=head1 NAME

Util::Object - Utils fr Objekte

=head1 METHODS 

=over

=item $objects_equals = Util::Object::equals($object_1_ref, $object_2_ref)

Liefert, ob zwei Objekte gleich sind. Gltige Objekte sind: Strings, Arrays
und Hashes. In Arrays und Hashes werden die einzelnen Elemente geprft, bei
Hashes auch die Elemente von Arrays und weiteren Hashes rekursiv. Die
Elemente in Arrays und Hashes mssen vom Typ gltiger Objekte sein (Strings,
Arrays, Hashes).

Der erste Parameter ist eine Referenz auf das 1. Objekt

Der zweite Parameter ist eine Referenz auf das 2. Objekt

Die Rckgabe ist true, wenn alle Elemente beider Objekte gleich sind

=item $is_type = Util::Object::is_type($type_string, $object)

Liefert, ob ein Objekt von einem bestimmten Typ ist.

Der 1. Parameter ist ein Typstring. Erlaubt sind: 'SCALAR', 'ARRAY', 'HASH',
'CODE', 'REF', 'GLOB' und 'LVALUE'.

Rckgabe ist true, wenn das Objekt vom spezifizierten Typ ist

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/10/14

=cut
