#!/usr/bin/perl -w

use strict;
use Test::More qw( no_plan );

use Util::Object;

test_equals();
test_is_type();

sub test_equals {
    my %hash_1 = ( a => 125, b => 250, c => 500 );
    my %hash_2 = ( a => 125, b => 250, c => 500 );
    my %hash_3 = ( a => 125, b => 150, c => 175 );

    ok( Util::Object::equals( \%hash_1,  \%hash_1 ) );
    ok( Util::Object::equals( \%hash_1,  \%hash_2 ) );
    ok( !Util::Object::equals( \%hash_1, \%hash_3 ) );

    my %hash_4 = ( a => 'Zoo', b => 'Affe', c => 'Giraffe' );
    my %hash_5 = ( a => 'Zoo', b => 'Affe', c => 'Giraffe' );
    my %hash_6 = ( a => 'Zoo', b => 'Affe', c => 'Lwe' );

    ok( Util::Object::equals( \%hash_4,  \%hash_4 ) );
    ok( Util::Object::equals( \%hash_4,  \%hash_5 ) );
    ok( !Util::Object::equals( \%hash_4, \%hash_6 ) );

    my %hash_7 = ();
    my %hash_8 = ();

    ok( Util::Object::equals( \%hash_7, \%hash_8 ) );

    my @array_1 = ( 'Telefon', 'Lautsprecher', 'Anrufbeantworter' );
    my @array_2 = ( 'Telefon', 'Lautsprecher', 'Anrufbeantworter' );
    my @array_3 = ( 'Telefon', 'Lautsprecher' );

    ok( Util::Object::equals( \@array_1,  \@array_1 ) );
    ok( Util::Object::equals( \@array_1,  \@array_2 ) );
    ok( !Util::Object::equals( \@array_1, \@array_3 ) );
    ok( !Util::Object::equals( \@array_1, \%hash_1 ) );
    ok( Util::Object::equals( undef,      undef ) );
    ok( !Util::Object::equals( \@array_1, undef ) );
    ok( !Util::Object::equals( @array_1,  @array_3 ) );

    my @array_4 = ();
    my @array_5 = ();

    ok( Util::Object::equals( \@array_4, \@array_5 ) );
    ok( !Util::Object::equals( \@array_4, \@array_1 ) );

    ok( Util::Object::equals( undef, undef ) );

    ok( !Util::Object::equals( \('Rainer'), \('Martin') ) );
    ok( Util::Object::equals( \('Martin'),  \('Martin') ) );
    ok( !Util::Object::equals( 'Martin',    'Martin' ) );
    ok( !Util::Object::equals( 12.5,        12.5 ) );
    ok( Util::Object::equals( \(12.5),      \(12.5) ) );
    ok( !Util::Object::equals( \(12.5),     \(12.6) ) );
}

sub test_is_type {
    my $scalar = 'Hallo';
    my @array  = ( 'A', 'B', 'C' );
    my %hash   = ( A => 'A', B => 'B', C => 'C' );

    ok( Util::Object::is_type( 'SCALAR', $scalar ) );
    ok( Util::Object::is_type( 'SCALAR', \$scalar ) );
    ok( Util::Object::is_type( 'ARRAY',  \@array ) );
    ok( Util::Object::is_type( 'HASH',   \%hash ) );
}
