#!/usr/bin/perl -w

use strict;
use Test::More qw( no_plan );

use XMP::Reader;
use Util::Array;

my $RES_DIR   = "$ENV{PROJECTDIR}/XmpIptc/t/xmp_files";
my %FILE_DATA = (
    "$RES_DIR/alles.xmp" => {
        'dc:subject'     => [ 'Haus', 'Landschaft', 'Strae' ],
        'dc:description' => ['Bildbeschreibung'],
        'dc:creator'     => ['Fotograf'],
        'dc:rights'      => ['Copyright'],
        'dc:title'       => ['Bezeichnung'],
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'Iptc4xmpCore:Location' => 'Ort',
        'photoshop:Headline'    => 'Wald bei Kirchheim'
    },
    "$RES_DIR/alles_umlaute.xmp" => {
        'dc:subject' => [ 'Haus', 'Landschaft', 'Strae', '' ],
        'dc:description'        => ['Bildbeschreibung '],
        'dc:creator'            => ['Fotograf '],
        'dc:rights'             => ['Copyright '],
        'dc:title'              => ['Bezeichnung '],
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'Iptc4xmpCore:Location' => 'Ort ',
        'photoshop:Headline'    => 'Wald bei Kirchheim'
    },
    "$RES_DIR/nur_bildbeschreibung.xmp" => {
        'dc:description' => [
            'Eine lngere Bildbeschreibung. Eine lngere Bildbeschreibung. Eine lngere Bildbeschreibung.'
        ],
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'photoshop:Headline'    => 'Wald bei Kirchheim'
    },
    "$RES_DIR/nur_copyright.xmp" => {
        'dc:rights'             => ['(c) 2007 by Elmar Baumann'],
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'photoshop:Headline'    => 'Wald bei Kirchheim'
    },
    "$RES_DIR/nur_fotograf.xmp" => {
        'dc:creator'            => ['Elmar Baumann <eb@elmar-baumann.de>'],
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'photoshop:Headline'    => 'Wald bei Kirchheim'
    },
    "$RES_DIR/nur_ort.xmp" => {
        'Iptc4xmpCore:Location' => 'Geroldshausen-Moos',
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'photoshop:Headline'    => 'Wald bei Kirchheim'
    },
    "$RES_DIR/ohne_iptc_meta.xmp" => {
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'photoshop:Headline'    => 'Wald bei Kirchheim'
    },
    "$RES_DIR/stichwoerter_drei.xmp" => {
        'dc:subject' => [ 'Herbst', 'Kursunlu Naturpark', 'Strae' ],
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'photoshop:Headline'    => 'Wald bei Kirchheim'
    },
    "$RES_DIR/stichwoerter_eines.xmp" => {
        'dc:subject'            => ['Strae'],
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'photoshop:Headline'    => 'Wald bei Kirchheim'
    },
    "$RES_DIR/komplett.xmp" => {
        'exif:DateTimeOriginal'    => '2007-10-24T18:17:37.90+02:00',
        'dc:subject'               => [ 'Herbst', 'Landschaft' ],
        'dc:creator'               => ['eb@elmar-baumann.de'],
        'dc:rights'                => ['Copyright (c) 2007 Elmar Baumann'],
        'dc:description'           => ['Bildbeschreibung'],
        'dc:title'                 => ['Bezeichnung'],
        'Iptc4xmpCore:Location'    => 'Ort',
        'Iptc4xmpCore:CountryCode' => 'ISO',
        'photoshop:Headline'       => 'IPTC-Titel',
        'photoshop:CaptionWriter'  => 'Autor der Beschreibung',
        'photoshop:Category'       => 'Kategorie',
        'photoshop:SupplementalCategories' => ['Andere Kategorien'],
        'photoshop:City'                   => 'Stadt',
        'photoshop:State'                  => 'Bundesland',
        'photoshop:Country'                => 'Land',
        'photoshop:TransmissionReference'  => 'Auftragskennung',
        'photoshop:Instructions'           => 'Anweisungen',
        'photoshop:Credit'                 => 'Anbieter',
        'photoshop:Source'                 => 'Quelle',
        'photoshop:AuthorsPosition'        => 'Position'
    },
    "$RES_DIR/alles-lightroom-1.3.xmp" => {
        'exif:DateTimeOriginal'    => '2007-11-18T17:19:19.00+01:00',
        'dc:subject'               => [ 'Stichwort-Tag 1', 'Stichwort-Tag 2', 'Stichwort-Tag 3' ],
        'dc:creator'               => ['Fotograf '],
        'dc:rights'                => ['Copyright'],
        'dc:description'           => ['Bildbeschreibung'],
        'dc:title'                 => ['Bezeichnung'],
        'Iptc4xmpCore:Location'    => 'Ort',
        'Iptc4xmpCore:CountryCode' => 'ISO',
        'photoshop:Headline'       => 'IPTC-Titel',
        'photoshop:CaptionWriter'  => 'Autor der Beschreibung',
        'photoshop:Category'       => 'Kategorie',
        'photoshop:SupplementalCategories' => ['Andere Kategorien'],
        'photoshop:City'                   => 'Stadt',
        'photoshop:State'                  => 'Bundesland/Kanton',
        'photoshop:Country'                => 'Land',
        'photoshop:TransmissionReference'  => 'Auftragskennung',
        'photoshop:Instructions'           => 'Anweisungen',
        'photoshop:Credit'                 => 'Anbieter',
        'photoshop:Source'                 => 'Quelle',
        'photoshop:AuthorsPosition'        => 'Position'
    },
);

test_nothing_read();
test_file_data();

sub test_file_data {
    my $xmp_reader = XMP::Reader->new();
    my @all_keys   = keys %main::XMP_INFO;

    foreach my $filename ( keys %FILE_DATA ) {
        $xmp_reader->read_file($filename);
        my $expected_hash_ref = $FILE_DATA{$filename};
        my $read_hash_ref     = $xmp_reader->get_xmp_data();
        foreach my $key (@all_keys) {
            my $read_value     = $read_hash_ref->{$key};
            my $expected_value = $expected_hash_ref->{$key};
            my $test_name      = "$filename [$key]";
            if ( defined $expected_value ) {
                if ( ref $read_value && ref $read_value eq 'ARRAY' ) {
                    ok( Util::Array::elements_equals(
                            $read_value, $expected_value
                        ),
                        $test_name
                    );
                }
                else {
                    is( $read_value, $expected_value, $test_name );
                }
            }
            else {
                is( $read_value, undef, $test_name );
            }
        }
    }
}

sub test_nothing_read {
    my $xmp_reader = XMP::Reader->new();
    my $xmp_data   = $xmp_reader->get_xmp_data();
    my @keys       = keys %main::XMP_INFO;

    foreach my $key (@keys) {
        my $test_name = "Nothing read [$key]";
        is( $xmp_data->{$key}, undef, $test_name );
    }
}
